SystemOrganization addCategory: #'SUnit-Utilities'!

Object subclass: #SUnitUnloader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Utilities'!

!SUnitUnloader commentStamp: 'LaurentLaffont 4/15/2011 20:20' prior: 0!
I am SUnitUnloader, a little tool to unload SUnit and a fixed set of Tests from the current image.

Usage: SUnitUnloader new unloadAll

Caveat: #unloadTestPackages uses a fixed and probably ill maintained list of what packages hold unit tests.
!

!SUnitUnloader methodsFor: 'private' stamp: 'MartinDias 2/15/2013 16:01'!
sortedForRemoveClasses: classesToRemove
	| removePlan remainingClasses |
	removePlan := OrderedCollection new.
	remainingClasses := classesToRemove as: OrderedCollection.

	[ remainingClasses isEmpty ]
		whileFalse: [ 
			| nextToRemove |
			nextToRemove := remainingClasses
				detect: [ :each | 
					| dependents |
					"get the classes that depend on me"
					dependents := each allSubclasses.	
					"I can be removed only if nobody else depends on me."
					(remainingClasses includesAny: dependents) not ].
			removePlan addLast: nextToRemove.
			remainingClasses remove: nextToRemove ].

	^ removePlan! !

!SUnitUnloader methodsFor: 'private' stamp: 'MartinDias 2/15/2013 15:59'!
sortedForRemovePackages: packagesToRemove
	| planToRemove remainingPackages |
	planToRemove := OrderedCollection new.
	remainingPackages := packagesToRemove as: OrderedCollection.

	[ remainingPackages isEmpty ]
		whileFalse: [ 
			| nextToRemove |
			nextToRemove := remainingPackages
				detect: [ :each | 
					| dependents |
					"get the packages that depend on me"
					dependents := (each definedClasses gather: [ :c | c allSubclasses collect: [ :s | s package ] ]) asSet.	
					"a package that depends on me shouldn't be removed after me."
					((remainingPackages copyWithout: each) includesAny: dependents) not ].
			planToRemove addLast: nextToRemove.
			remainingPackages remove: nextToRemove ].

	^ planToRemove! !

!SUnitUnloader methodsFor: 'private' stamp: 'EstebanLorenzano 2/19/2013 15:51'!
sunitPackages
	"Answers the SUnit-UI package, not all the infraestructure"
	^ Array 
		with: (RPackage organizer packageNamed:'SUnit-UI')
		with: (RPackage organizer packageNamed:'SUnit-Utilities')
! !

!SUnitUnloader methodsFor: 'unloading' stamp: 'MartinDias 2/15/2013 16:42'!
testPackages

	^(TestCase allSubclasses collect: [ :c | c package ] as: Set)
		select: [ :p | 
			(p name matchesRegex: '.*Test.*') 
			and: [ (p name beginsWith: 'SUnit') not ] ].! !

!SUnitUnloader methodsFor: 'unloading' stamp: 'EstebanLorenzano 2/19/2013 15:35'!
unloadAll
	| packagesToRemove |
	packagesToRemove := self testPackages.
	self unloadTestCases.
	self unloadPackages: packagesToRemove.
	self unloadSUnit.! !

!SUnitUnloader methodsFor: 'private' stamp: 'MartinDias 2/15/2013 16:39'!
unloadPackages: packagesToRemove
	
	(self sortedForRemovePackages: packagesToRemove) 
		do: [:aPackage |  (MCPackage named: aPackage name) unload ].! !

!SUnitUnloader methodsFor: 'private' stamp: 'EstebanLorenzano 2/19/2013 15:52'!
unloadSUnit
	Smalltalk globals
		at: #TestCase
		ifPresent: [ :class | SystemAnnouncer uniqueInstance unsubscribe: class ].

	Smalltalk tools remove: #testRunner.	
		
	(self sortedForRemovePackages: self sunitPackages) 
		do: [ :each | (MCPackage named: each name) unload ].
	
	AppRegistry removeObsolete.! !

!SUnitUnloader methodsFor: 'unloading' stamp: 'MartinDias 2/15/2013 16:30'!
unloadTestCases
	
	| classesToRemove sunitPackages |
	sunitPackages := self sunitPackages.
	classesToRemove := TestCase allSubclasses
		reject: [:aClass | sunitPackages includes: aClass package ].

	(self sortedForRemoveClasses: classesToRemove) 
		do: [:aClass | aClass removeFromSystem ].! !

!SUnitUnloader methodsFor: 'unloading' stamp: 'MartinDias 2/15/2013 16:31'!
unloadTestPackages

	| packagesToRemove |
	packagesToRemove := (TestCase allSubclasses collect: [ :c | c package ] as: Set)
		select: [ :p | (p name matchesRegex: '.*Test.*') and: [ (p name beginsWith: 'SUnit') not ] ].
	
	(self sortedForRemovePackages: packagesToRemove) 
		do: [:aPackage |  (MCPackage named: aPackage name) unload ].! !
